// Code generated by "stringer -type=DatumKind -linecomment"; DO NOT EDIT.

//go:build go1.18

package compute

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[KindNone-0]
	_ = x[KindScalar-1]
	_ = x[KindArray-2]
	_ = x[KindChunked-3]
	_ = x[KindRecord-4]
	_ = x[KindTable-5]
}

const _DatumKind_name = "nonescalararraychunked_arrayrecord_batchtable"

var _DatumKind_index = [...]uint8{0, 4, 10, 15, 28, 40, 45}

func (i DatumKind) String() string {
	if i < 0 || i >= DatumKind(len(_DatumKind_index)-1) {
		return "DatumKind(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _DatumKind_name[_DatumKind_index[i]:_DatumKind_index[i+1]]
}
