# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from wandb._pydantic import GQLResult


class UpdateUserRegistryRole(GQLResult):
    result: Optional[UpdateUserRegistryRoleResult]


class UpdateUserRegistryRoleResult(GQLResult):
    success: bool


UpdateUserRegistryRole.model_rebuild()
