# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from wandb._pydantic import GQLResult


class AddAliases(GQLResult):
    result: Optional[AddAliasesResult]


class AddAliasesResult(GQLResult):
    success: bool


AddAliases.model_rebuild()
