# Generated by ariadne-codegen

__all__ = ["SERVER_FEATURES_QUERY_GQL", "ServerFeaturesQuery"]
from .operations import SERVER_FEATURES_QUERY_GQL
from .server_features_query import ServerFeaturesQuery
