# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from wandb._pydantic import GQLResult

from .fragments import RegistryFragment


class FetchRegistry(GQLResult):
    entity: Optional[FetchRegistryEntity]


class FetchRegistryEntity(GQLResult):
    project: Optional[RegistryFragment]


FetchRegistry.model_rebuild()
FetchRegistryEntity.model_rebuild()
