# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLResult


class DeleteRegistry(GQLResult):
    delete_model: Optional[DeleteRegistryDeleteModel] = Field(alias="deleteModel")


class DeleteRegistryDeleteModel(GQLResult):
    success: Optional[bool]


DeleteRegistry.model_rebuild()
