# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import List, Optional

from wandb._pydantic import GQLResult

from .fragments import TagFragment


class AddArtifactCollectionTags(GQLResult):
    result: Optional[AddArtifactCollectionTagsResult]


class AddArtifactCollectionTagsResult(GQLResult):
    tags: List[TagFragment]


AddArtifactCollectionTags.model_rebuild()
AddArtifactCollectionTagsResult.model_rebuild()
