# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import List, Optional

from pydantic import Field

from wandb._pydantic import GQLResult

from .fragments import FileWithUrlFragment, PageInfoFragment


class GetArtifactFileUrls(GQLResult):
    artifact: Optional[GetArtifactFileUrlsArtifact]


class GetArtifactFileUrlsArtifact(GQLResult):
    files: Optional[GetArtifactFileUrlsArtifactFiles]


class GetArtifactFileUrlsArtifactFiles(GQLResult):
    page_info: PageInfoFragment = Field(alias="pageInfo")
    edges: List[GetArtifactFileUrlsArtifactFilesEdges]


class GetArtifactFileUrlsArtifactFilesEdges(GQLResult):
    node: Optional[FileWithUrlFragment]


GetArtifactFileUrls.model_rebuild()
GetArtifactFileUrlsArtifact.model_rebuild()
GetArtifactFileUrlsArtifactFiles.model_rebuild()
GetArtifactFileUrlsArtifactFilesEdges.model_rebuild()
