// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

//go:build !noasm
// +build !noasm

package utils

import "unsafe"

//go:noescape
func _bytes_to_bools_avx2(in unsafe.Pointer, len int, out unsafe.Pointer, outlen int)

// use AVX2 to SIMD accelerate the conversion of bitmap to boolean slice
func bytesToBoolsAVX2(in []byte, out []bool) {
	_bytes_to_bools_avx2(unsafe.Pointer(&in[0]), len(in), unsafe.Pointer(&out[0]), len(out))
}
