# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: wandb/proto/wandb_sync.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from wandb.proto import wandb_internal_pb2 as wandb_dot_proto_dot_wandb__internal__pb2
from wandb.proto import wandb_settings_pb2 as wandb_dot_proto_dot_wandb__settings__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1cwandb/proto/wandb_sync.proto\x12\x0ewandb_internal\x1a wandb/proto/wandb_internal.proto\x1a wandb/proto/wandb_settings.proto\"Q\n\x15ServerInitSyncRequest\x12\x0c\n\x04path\x18\x01 \x03(\t\x12*\n\x08settings\x18\x02 \x01(\x0b\x32\x18.wandb_internal.Settings\"$\n\x16ServerInitSyncResponse\x12\n\n\x02id\x18\x01 \x01(\t\"4\n\x11ServerSyncRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x13\n\x0bparallelism\x18\x02 \x01(\r\"I\n\x12ServerSyncResponse\x12\x33\n\x08messages\x18\x01 \x03(\x0b\x32!.wandb_internal.ServerSyncMessage\"%\n\x17ServerSyncStatusRequest\x12\n\n\x02id\x18\x01 \x01(\t\"\xe5\x01\n\x18ServerSyncStatusResponse\x12\x42\n\x05stats\x18\x01 \x03(\x0b\x32\x33.wandb_internal.ServerSyncStatusResponse.StatsEntry\x12\x37\n\x0cnew_messages\x18\x02 \x03(\x0b\x32!.wandb_internal.ServerSyncMessage\x1aL\n\nStatsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12-\n\x05value\x18\x02 \x01(\x0b\x32\x1e.wandb_internal.OperationStats:\x02\x38\x01\"\xaa\x01\n\x11ServerSyncMessage\x12<\n\x08severity\x18\x01 \x01(\x0e\x32*.wandb_internal.ServerSyncMessage.Severity\x12\x0f\n\x07\x63ontent\x18\x02 \x01(\t\"F\n\x08Severity\x12\x13\n\x0fSEVERITY_NOTSET\x10\x00\x12\x11\n\rSEVERITY_INFO\x10\x14\x12\x12\n\x0eSEVERITY_ERROR\x10(B\x1bZ\x19\x63ore/pkg/service_go_protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'wandb.proto.wandb_sync_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\031core/pkg/service_go_proto'
  _SERVERSYNCSTATUSRESPONSE_STATSENTRY._options = None
  _SERVERSYNCSTATUSRESPONSE_STATSENTRY._serialized_options = b'8\001'
  _SERVERINITSYNCREQUEST._serialized_start=116
  _SERVERINITSYNCREQUEST._serialized_end=197
  _SERVERINITSYNCRESPONSE._serialized_start=199
  _SERVERINITSYNCRESPONSE._serialized_end=235
  _SERVERSYNCREQUEST._serialized_start=237
  _SERVERSYNCREQUEST._serialized_end=289
  _SERVERSYNCRESPONSE._serialized_start=291
  _SERVERSYNCRESPONSE._serialized_end=364
  _SERVERSYNCSTATUSREQUEST._serialized_start=366
  _SERVERSYNCSTATUSREQUEST._serialized_end=403
  _SERVERSYNCSTATUSRESPONSE._serialized_start=406
  _SERVERSYNCSTATUSRESPONSE._serialized_end=635
  _SERVERSYNCSTATUSRESPONSE_STATSENTRY._serialized_start=559
  _SERVERSYNCSTATUSRESPONSE_STATSENTRY._serialized_end=635
  _SERVERSYNCMESSAGE._serialized_start=638
  _SERVERSYNCMESSAGE._serialized_end=808
  _SERVERSYNCMESSAGE_SEVERITY._serialized_start=738
  _SERVERSYNCMESSAGE_SEVERITY._serialized_end=808
# @@protoc_insertion_point(module_scope)
