# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from wandb._pydantic import GQLResult

from .fragments import ArtifactFragment


class ArtifactByID(GQLResult):
    artifact: Optional[ArtifactFragment]


ArtifactByID.model_rebuild()
