# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLResult

from .fragments import ArtifactTypeFragment


class ProjectArtifactType(GQLResult):
    project: Optional[ProjectArtifactTypeProject]


class ProjectArtifactTypeProject(GQLResult):
    artifact_type: Optional[ArtifactTypeFragment] = Field(alias="artifactType")


ProjectArtifactType.model_rebuild()
ProjectArtifactTypeProject.model_rebuild()
