# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from wandb._pydantic import GQLResult


class CreateRegistryMembers(GQLResult):
    result: Optional[CreateRegistryMembersResult]


class CreateRegistryMembersResult(GQLResult):
    success: bool


CreateRegistryMembers.model_rebuild()
