# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLResult

from .fragments import TypeInfoFragment


class TypeInfo(GQLResult):
    type: Optional[TypeInfoFragment] = Field(alias="__type")


TypeInfo.model_rebuild()
