# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import List, Optional

from pydantic import Field

from wandb._pydantic import GQLResult

from .fragments import ArtifactFragment, PageInfoFragment


class RunInputArtifacts(GQLResult):
    project: Optional[RunInputArtifactsProject]


class RunInputArtifactsProject(GQLResult):
    run: Optional[RunInputArtifactsProjectRun]


class RunInputArtifactsProjectRun(GQLResult):
    artifacts: Optional[RunInputArtifactsProjectRunArtifacts]


class RunInputArtifactsProjectRunArtifacts(GQLResult):
    total_count: int = Field(alias="totalCount")
    page_info: PageInfoFragment = Field(alias="pageInfo")
    edges: List[RunInputArtifactsProjectRunArtifactsEdges]


class RunInputArtifactsProjectRunArtifactsEdges(GQLResult):
    node: Optional[ArtifactFragment]


RunInputArtifacts.model_rebuild()
RunInputArtifactsProject.model_rebuild()
RunInputArtifactsProjectRun.model_rebuild()
RunInputArtifactsProjectRunArtifacts.model_rebuild()
RunInputArtifactsProjectRunArtifactsEdges.model_rebuild()
