# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import Optional

from pydantic import Field

from wandb._pydantic import GQLResult

from .fragments import ArtifactCollectionFragment


class ProjectArtifactCollection(GQLResult):
    project: Optional[ProjectArtifactCollectionProject]


class ProjectArtifactCollectionProject(GQLResult):
    artifact_type: Optional[ProjectArtifactCollectionProjectArtifactType] = Field(
        alias="artifactType"
    )


class ProjectArtifactCollectionProjectArtifactType(GQLResult):
    artifact_collection: Optional[ArtifactCollectionFragment] = Field(
        alias="artifactCollection"
    )


ProjectArtifactCollection.model_rebuild()
ProjectArtifactCollectionProject.model_rebuild()
ProjectArtifactCollectionProjectArtifactType.model_rebuild()
