# Generated by ariadne-codegen
# Source: tools/graphql_codegen/artifacts/

from __future__ import annotations

from typing import List, Optional

from pydantic import Field

from wandb._pydantic import GQLResult

from .fragments import RunInfoFragment


class ArtifactUsedBy(GQLResult):
    artifact: Optional[ArtifactUsedByArtifact]


class ArtifactUsedByArtifact(GQLResult):
    used_by: ArtifactUsedByArtifactUsedBy = Field(alias="usedBy")


class ArtifactUsedByArtifactUsedBy(GQLResult):
    edges: List[ArtifactUsedByArtifactUsedByEdges]


class ArtifactUsedByArtifactUsedByEdges(GQLResult):
    node: RunInfoFragment


ArtifactUsedBy.model_rebuild()
ArtifactUsedByArtifact.model_rebuild()
ArtifactUsedByArtifactUsedBy.model_rebuild()
ArtifactUsedByArtifactUsedByEdges.model_rebuild()
